from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ...models.o_data_errors.o_data_error import ODataError
    from ...models.permissions_management import PermissionsManagement
    from .permissions_request_changes.permissions_request_changes_request_builder import PermissionsRequestChangesRequestBuilder
    from .scheduled_permissions_approvals.scheduled_permissions_approvals_request_builder import ScheduledPermissionsApprovalsRequestBuilder
    from .scheduled_permissions_requests.scheduled_permissions_requests_request_builder import ScheduledPermissionsRequestsRequestBuilder

class PermissionsManagementRequestBuilder(BaseRequestBuilder):
    """
    Provides operations to manage the permissionsManagement property of the microsoft.graph.identityGovernance entity.
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new PermissionsManagementRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/identityGovernance/permissionsManagement{?%24expand,%24select}", path_parameters)
    
    async def delete(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> None:
        """
        Delete navigation property permissionsManagement for identityGovernance
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: None
        """
        request_info = self.to_delete_request_information(
            request_configuration
        )
        from ...models.o_data_errors.o_data_error import ODataError

        error_mapping: Dict[str, ParsableFactory] = {
            "XXX": ODataError,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        return await self.request_adapter.send_no_response_content_async(request_info, error_mapping)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[PermissionsManagementRequestBuilderGetQueryParameters]] = None) -> Optional[PermissionsManagement]:
        """
        Get permissionsManagement from identityGovernance
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[PermissionsManagement]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        from ...models.o_data_errors.o_data_error import ODataError

        error_mapping: Dict[str, ParsableFactory] = {
            "XXX": ODataError,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ...models.permissions_management import PermissionsManagement

        return await self.request_adapter.send_async(request_info, PermissionsManagement, error_mapping)
    
    async def patch(self,body: PermissionsManagement, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[PermissionsManagement]:
        """
        Update the navigation property permissionsManagement in identityGovernance
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[PermissionsManagement]
        """
        if not body:
            raise TypeError("body cannot be null.")
        request_info = self.to_patch_request_information(
            body, request_configuration
        )
        from ...models.o_data_errors.o_data_error import ODataError

        error_mapping: Dict[str, ParsableFactory] = {
            "XXX": ODataError,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ...models.permissions_management import PermissionsManagement

        return await self.request_adapter.send_async(request_info, PermissionsManagement, error_mapping)
    
    def to_delete_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Delete navigation property permissionsManagement for identityGovernance
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.DELETE, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[PermissionsManagementRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Get permissionsManagement from identityGovernance
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_patch_request_information(self,body: PermissionsManagement, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Update the navigation property permissionsManagement in identityGovernance
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if not body:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.PATCH, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> PermissionsManagementRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: PermissionsManagementRequestBuilder
        """
        if not raw_url:
            raise TypeError("raw_url cannot be null.")
        return PermissionsManagementRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def permissions_request_changes(self) -> PermissionsRequestChangesRequestBuilder:
        """
        Provides operations to manage the permissionsRequestChanges property of the microsoft.graph.permissionsManagement entity.
        """
        from .permissions_request_changes.permissions_request_changes_request_builder import PermissionsRequestChangesRequestBuilder

        return PermissionsRequestChangesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def scheduled_permissions_approvals(self) -> ScheduledPermissionsApprovalsRequestBuilder:
        """
        Provides operations to manage the scheduledPermissionsApprovals property of the microsoft.graph.permissionsManagement entity.
        """
        from .scheduled_permissions_approvals.scheduled_permissions_approvals_request_builder import ScheduledPermissionsApprovalsRequestBuilder

        return ScheduledPermissionsApprovalsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def scheduled_permissions_requests(self) -> ScheduledPermissionsRequestsRequestBuilder:
        """
        Provides operations to manage the scheduledPermissionsRequests property of the microsoft.graph.permissionsManagement entity.
        """
        from .scheduled_permissions_requests.scheduled_permissions_requests_request_builder import ScheduledPermissionsRequestsRequestBuilder

        return ScheduledPermissionsRequestsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class PermissionsManagementRequestBuilderDeleteRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class PermissionsManagementRequestBuilderGetQueryParameters():
        """
        Get permissionsManagement from identityGovernance
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if not original_name:
                raise TypeError("original_name cannot be null.")
            if original_name == "expand":
                return "%24expand"
            if original_name == "select":
                return "%24select"
            return original_name
        
        # Expand related entities
        expand: Optional[List[str]] = None

        # Select properties to be returned
        select: Optional[List[str]] = None

    
    @dataclass
    class PermissionsManagementRequestBuilderGetRequestConfiguration(RequestConfiguration[PermissionsManagementRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class PermissionsManagementRequestBuilderPatchRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    

