from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from kiota_abstractions.store import BackedModel, BackingStore, BackingStoreFactorySingleton
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .management_template_detailed_info import ManagementTemplateDetailedInfo

@dataclass
class ManagementIntentInfo(AdditionalDataHolder, BackedModel, Parsable):
    # Stores model information.
    backing_store: BackingStore = field(default_factory=BackingStoreFactorySingleton(backing_store_factory=None).backing_store_factory.create_backing_store, repr=False)

    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)
    # The display name for the management intent. Optional. Read-only.
    management_intent_display_name: Optional[str] = None
    # The identifier for the management intent. Required. Read-only.
    management_intent_id: Optional[str] = None
    # The collection of management template information associated with the management intent. Optional. Read-only.
    management_templates: Optional[List[ManagementTemplateDetailedInfo]] = None
    # The OdataType property
    odata_type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ManagementIntentInfo:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ManagementIntentInfo
        """
        if not parse_node:
            raise TypeError("parse_node cannot be null.")
        return ManagementIntentInfo()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .management_template_detailed_info import ManagementTemplateDetailedInfo

        from .management_template_detailed_info import ManagementTemplateDetailedInfo

        fields: Dict[str, Callable[[Any], None]] = {
            "managementIntentDisplayName": lambda n : setattr(self, 'management_intent_display_name', n.get_str_value()),
            "managementIntentId": lambda n : setattr(self, 'management_intent_id', n.get_str_value()),
            "managementTemplates": lambda n : setattr(self, 'management_templates', n.get_collection_of_object_values(ManagementTemplateDetailedInfo)),
            "@odata.type": lambda n : setattr(self, 'odata_type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if not writer:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("managementIntentDisplayName", self.management_intent_display_name)
        writer.write_str_value("managementIntentId", self.management_intent_id)
        writer.write_collection_of_object_values("managementTemplates", self.management_templates)
        writer.write_str_value("@odata.type", self.odata_type)
        writer.write_additional_data_value(self.additional_data)
    

