from __future__ import annotations
import datetime
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item_facet import ItemFacet

from .item_facet import ItemFacet

@dataclass
class PersonCertification(ItemFacet):
    # The OdataType property
    odata_type: Optional[str] = "#microsoft.graph.personCertification"
    # The referenceable identifier for the certification.
    certification_id: Optional[str] = None
    # Description of the certification.
    description: Optional[str] = None
    # Title of the certification.
    display_name: Optional[str] = None
    # The date that the certification expires.
    end_date: Optional[datetime.date] = None
    # The date that the certification was issued.
    issued_date: Optional[datetime.date] = None
    # Authority which granted the certification.
    issuing_authority: Optional[str] = None
    # Company which granted the certification.
    issuing_company: Optional[str] = None
    # The date that the certification became valid.
    start_date: Optional[datetime.date] = None
    # URL referencing a thumbnail of the certification.
    thumbnail_url: Optional[str] = None
    # URL referencing the certification.
    web_url: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PersonCertification:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PersonCertification
        """
        if not parse_node:
            raise TypeError("parse_node cannot be null.")
        return PersonCertification()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .item_facet import ItemFacet

        from .item_facet import ItemFacet

        fields: Dict[str, Callable[[Any], None]] = {
            "certificationId": lambda n : setattr(self, 'certification_id', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "displayName": lambda n : setattr(self, 'display_name', n.get_str_value()),
            "endDate": lambda n : setattr(self, 'end_date', n.get_date_value()),
            "issuedDate": lambda n : setattr(self, 'issued_date', n.get_date_value()),
            "issuingAuthority": lambda n : setattr(self, 'issuing_authority', n.get_str_value()),
            "issuingCompany": lambda n : setattr(self, 'issuing_company', n.get_str_value()),
            "startDate": lambda n : setattr(self, 'start_date', n.get_date_value()),
            "thumbnailUrl": lambda n : setattr(self, 'thumbnail_url', n.get_str_value()),
            "webUrl": lambda n : setattr(self, 'web_url', n.get_str_value()),
        }
        super_fields = super().get_field_deserializers()
        fields.update(super_fields)
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if not writer:
            raise TypeError("writer cannot be null.")
        super().serialize(writer)
        writer.write_str_value("certificationId", self.certification_id)
        writer.write_str_value("description", self.description)
        writer.write_str_value("displayName", self.display_name)
        writer.write_date_value("endDate", self.end_date)
        writer.write_date_value("issuedDate", self.issued_date)
        writer.write_str_value("issuingAuthority", self.issuing_authority)
        writer.write_str_value("issuingCompany", self.issuing_company)
        writer.write_date_value("startDate", self.start_date)
        writer.write_str_value("thumbnailUrl", self.thumbnail_url)
        writer.write_str_value("webUrl", self.web_url)
    

