from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ......models.o_data_errors.o_data_error import ODataError
    from ......models.security.sensitivity_label import SensitivityLabel
    from ......models.security.sensitivity_label_collection_response import SensitivityLabelCollectionResponse
    from .count.count_request_builder import CountRequestBuilder
    from .item.sensitivity_label_item_request_builder import SensitivityLabelItemRequestBuilder
    from .microsoft_graph_security_evaluate_application.microsoft_graph_security_evaluate_application_request_builder import MicrosoftGraphSecurityEvaluateApplicationRequestBuilder
    from .microsoft_graph_security_evaluate_classification_results.microsoft_graph_security_evaluate_classification_results_request_builder import MicrosoftGraphSecurityEvaluateClassificationResultsRequestBuilder
    from .microsoft_graph_security_evaluate_removal.microsoft_graph_security_evaluate_removal_request_builder import MicrosoftGraphSecurityEvaluateRemovalRequestBuilder
    from .microsoft_graph_security_extract_content_label.microsoft_graph_security_extract_content_label_request_builder import MicrosoftGraphSecurityExtractContentLabelRequestBuilder

class SensitivityLabelsRequestBuilder(BaseRequestBuilder):
    """
    Provides operations to manage the sensitivityLabels property of the microsoft.graph.security.informationProtection entity.
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new SensitivityLabelsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/users/{user%2Did}/security/informationProtection/sensitivityLabels{?%24count,%24expand,%24filter,%24orderby,%24search,%24select,%24skip,%24top}", path_parameters)
    
    def by_sensitivity_label_id(self,sensitivity_label_id: str) -> SensitivityLabelItemRequestBuilder:
        """
        Provides operations to manage the sensitivityLabels property of the microsoft.graph.security.informationProtection entity.
        param sensitivity_label_id: The unique identifier of sensitivityLabel
        Returns: SensitivityLabelItemRequestBuilder
        """
        if not sensitivity_label_id:
            raise TypeError("sensitivity_label_id cannot be null.")
        from .item.sensitivity_label_item_request_builder import SensitivityLabelItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["sensitivityLabel%2Did"] = sensitivity_label_id
        return SensitivityLabelItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[SensitivityLabelsRequestBuilderGetQueryParameters]] = None) -> Optional[SensitivityLabelCollectionResponse]:
        """
        Get a list of sensitivityLabel objects associated with a user or organization.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[SensitivityLabelCollectionResponse]
        Find more info here: https://learn.microsoft.com/graph/api/security-informationprotection-list-sensitivitylabels?view=graph-rest-beta
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        from ......models.o_data_errors.o_data_error import ODataError

        error_mapping: Dict[str, ParsableFactory] = {
            "XXX": ODataError,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ......models.security.sensitivity_label_collection_response import SensitivityLabelCollectionResponse

        return await self.request_adapter.send_async(request_info, SensitivityLabelCollectionResponse, error_mapping)
    
    async def post(self,body: SensitivityLabel, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[SensitivityLabel]:
        """
        Create new navigation property to sensitivityLabels for users
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[SensitivityLabel]
        """
        if not body:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        from ......models.o_data_errors.o_data_error import ODataError

        error_mapping: Dict[str, ParsableFactory] = {
            "XXX": ODataError,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ......models.security.sensitivity_label import SensitivityLabel

        return await self.request_adapter.send_async(request_info, SensitivityLabel, error_mapping)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[SensitivityLabelsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Get a list of sensitivityLabel objects associated with a user or organization.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: SensitivityLabel, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Create new navigation property to sensitivityLabels for users
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if not body:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> SensitivityLabelsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: SensitivityLabelsRequestBuilder
        """
        if not raw_url:
            raise TypeError("raw_url cannot be null.")
        return SensitivityLabelsRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def count(self) -> CountRequestBuilder:
        """
        Provides operations to count the resources in the collection.
        """
        from .count.count_request_builder import CountRequestBuilder

        return CountRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def microsoft_graph_security_evaluate_application(self) -> MicrosoftGraphSecurityEvaluateApplicationRequestBuilder:
        """
        Provides operations to call the evaluateApplication method.
        """
        from .microsoft_graph_security_evaluate_application.microsoft_graph_security_evaluate_application_request_builder import MicrosoftGraphSecurityEvaluateApplicationRequestBuilder

        return MicrosoftGraphSecurityEvaluateApplicationRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def microsoft_graph_security_evaluate_classification_results(self) -> MicrosoftGraphSecurityEvaluateClassificationResultsRequestBuilder:
        """
        Provides operations to call the evaluateClassificationResults method.
        """
        from .microsoft_graph_security_evaluate_classification_results.microsoft_graph_security_evaluate_classification_results_request_builder import MicrosoftGraphSecurityEvaluateClassificationResultsRequestBuilder

        return MicrosoftGraphSecurityEvaluateClassificationResultsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def microsoft_graph_security_evaluate_removal(self) -> MicrosoftGraphSecurityEvaluateRemovalRequestBuilder:
        """
        Provides operations to call the evaluateRemoval method.
        """
        from .microsoft_graph_security_evaluate_removal.microsoft_graph_security_evaluate_removal_request_builder import MicrosoftGraphSecurityEvaluateRemovalRequestBuilder

        return MicrosoftGraphSecurityEvaluateRemovalRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def microsoft_graph_security_extract_content_label(self) -> MicrosoftGraphSecurityExtractContentLabelRequestBuilder:
        """
        Provides operations to call the extractContentLabel method.
        """
        from .microsoft_graph_security_extract_content_label.microsoft_graph_security_extract_content_label_request_builder import MicrosoftGraphSecurityExtractContentLabelRequestBuilder

        return MicrosoftGraphSecurityExtractContentLabelRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class SensitivityLabelsRequestBuilderGetQueryParameters():
        """
        Get a list of sensitivityLabel objects associated with a user or organization.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if not original_name:
                raise TypeError("original_name cannot be null.")
            if original_name == "count":
                return "%24count"
            if original_name == "expand":
                return "%24expand"
            if original_name == "filter":
                return "%24filter"
            if original_name == "orderby":
                return "%24orderby"
            if original_name == "search":
                return "%24search"
            if original_name == "select":
                return "%24select"
            if original_name == "skip":
                return "%24skip"
            if original_name == "top":
                return "%24top"
            return original_name
        
        # Include count of items
        count: Optional[bool] = None

        # Expand related entities
        expand: Optional[List[str]] = None

        # Filter items by property values
        filter: Optional[str] = None

        # Order items by property values
        orderby: Optional[List[str]] = None

        # Search items by search phrases
        search: Optional[str] = None

        # Select properties to be returned
        select: Optional[List[str]] = None

        # Skip the first n items
        skip: Optional[int] = None

        # Show only the first n items
        top: Optional[int] = None

    
    @dataclass
    class SensitivityLabelsRequestBuilderGetRequestConfiguration(RequestConfiguration[SensitivityLabelsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class SensitivityLabelsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    

